@@ -1,11 +1,18 @@
+/// code to tickle various supervisors to enable the `zeronsd supervise` command.
+/// this code is hard to read but fundamentally launchd and systemd are controlled through a
+/// library called `tinytemplate` and of course serde.
 use std::path::PathBuf;
 
 use anyhow::anyhow;
 use log::info;
+use regex::Regex;
 use serde::Serialize;
 use tinytemplate::TinyTemplate;
 use trust_dns_resolver::Name;
 
+#[cfg(target_os = "linux")]
+use std::os::unix::fs::PermissionsExt;
+
 #[cfg(target_os = "windows")]
 const SUPERVISE_SYSTEM_DIR: &str = "";
 #[cfg(target_os = "windows")]
@@ -13,9 +20,11 @@ const SERVICE_TEMPLATE: &str = "";
 
 #[cfg(target_os = "linux")]
 const SUPERVISE_SYSTEM_DIR: &str = "/lib/systemd/system";
+#[cfg(target_os = "linux")]
+const OS_RELEASE_FILE: &str = "/etc/os-release";
 
 #[cfg(target_os = "linux")]
-const SERVICE_TEMPLATE: &str = "
+const SYSTEMD_TEMPLATE: &str = "
 [Unit]
 Description=zeronsd for network {network}
 Requires=zerotier-one.service
@@ -30,6 +39,24 @@ TimeoutStopSec=30
 WantedBy=default.target
 ";
 
+#[cfg(target_os = "linux")]
+const ALPINE_INIT_DIR: &str = "/etc/init.d";
+#[cfg(target_os = "linux")]
+const ALPINE_TEMPLATE: &str = r#"
+#!/sbin/openrc-run
+
+depend() \{
+    need zerotier-one
+    use network dns logger netmount
+}
+
+description="zeronsd for network {network}"
+command="{binpath}"
+command_args="start -t {token} {{ if wildcard_names }}-w {{endif}}{{ if authtoken }}-s {authtoken} {{endif}}{{ if hosts_file }}-f {hosts_file} {{ endif }}{{ if domain }}-d {domain} {{ endif }}{network}"
+command_background="yes"
+pidfile="/run/$RC_SVCNAME.pid"
+"#;
+
 #[cfg(target_os = "macos")]
 const SUPERVISE_SYSTEM_DIR: &str = "/Library/LaunchDaemons/";
 #[cfg(target_os = "macos")]
@@ -86,6 +113,7 @@ pub struct Properties {
     pub authtoken: Option<String>,
     pub token: String,
     pub wildcard_names: bool,
+    pub distro: Option<String>,
 }
 
 impl From<&clap::ArgMatches<'_>> for Properties {
@@ -112,6 +140,7 @@ impl Default for Properties {
             hosts_file: None,
             authtoken: None,
             token: String::new(),
+            distro: None,
         }
     }
 }
@@ -125,7 +154,27 @@ impl<'a> Properties {
         token: Option<&'_ str>,
         wildcard_names: bool,
     ) -> Result<Self, anyhow::Error> {
+        let distro = if cfg!(target_os = "linux") {
+            if let Ok(release) = std::fs::read_to_string(OS_RELEASE_FILE) {
+                let id_regex = Regex::new(r#"\nID=(.+)\n"#)?;
+                if let Some(caps) = id_regex.captures(&release) {
+                    if let Some(distro) = caps.get(1) {
+                        Some(distro.clone().as_str().to_string())
+                    } else {
+                        None
+                    }
+                } else {
+                    return Err(anyhow!("Could not determine Linux distribution; you'll need to configure supervision manually. Sorry!"));
+                }
+            } else {
+                return Err(anyhow!("Could not determine Linux distribution; you'll need to configure supervision manually. Sorry!"));
+            }
+        } else {
+            None
+        };
+
         Ok(Self {
+            distro,
             wildcard_names,
             binpath: String::from(std::env::current_exe()?.to_string_lossy()),
             // make this garbage a macro later
@@ -217,14 +266,37 @@ impl<'a> Properties {
     }
 
     pub fn supervise_template(&self) -> Result<String, anyhow::Error> {
+        let template = self.get_service_template();
+
         let mut t = TinyTemplate::new();
-        t.add_template("supervise", SERVICE_TEMPLATE)?;
+        t.add_template("supervise", template)?;
         match t.render("supervise", self) {
             Ok(x) => Ok(x),
             Err(e) => Err(anyhow!(e)),
         }
     }
 
+    #[cfg(target_os = "linux")]
+    fn get_service_template(&self) -> &str {
+        match self.distro.clone() {
+            Some(s) => match s.as_str() {
+                "alpine" => ALPINE_TEMPLATE.trim(),
+                _ => SYSTEMD_TEMPLATE,
+            },
+            None => SYSTEMD_TEMPLATE,
+        }
+    }
+
+    #[cfg(target_os = "windows")]
+    fn get_service_template(&self) -> &str {
+        return SERVICE_TEMPLATE;
+    }
+
+    #[cfg(target_os = "macos")]
+    fn get_service_template(&self) -> &str {
+        return SERVICE_TEMPLATE;
+    }
+
     #[cfg(target_os = "windows")]
     fn service_name(&self) -> String {
         return String::new();
@@ -232,7 +304,13 @@ impl<'a> Properties {
 
     #[cfg(target_os = "linux")]
     fn service_name(&self) -> String {
-        format!("zeronsd-{}.service", self.network)
+        match self.distro.clone() {
+            Some(s) => match s.as_str() {
+                "alpine" => format!("zeronsd-{}", self.network),
+                _ => format!("zeronsd-{}.service", self.network),
+            },
+            None => format!("zeronsd-{}.service", self.network),
+        }
     }
 
     #[cfg(target_os = "macos")]
@@ -241,13 +319,29 @@ impl<'a> Properties {
     }
 
     fn service_path(&self) -> PathBuf {
-        PathBuf::from(SUPERVISE_SYSTEM_DIR).join(self.service_name())
+        let dir = match self.distro.clone() {
+            Some(s) => match s.as_str() {
+                "alpine" => ALPINE_INIT_DIR,
+                _ => SUPERVISE_SYSTEM_DIR,
+            },
+            None => SUPERVISE_SYSTEM_DIR,
+        };
+        PathBuf::from(dir).join(self.service_name())
     }
 
     pub fn install_supervisor(&mut self) -> Result<(), anyhow::Error> {
         self.validate()?;
 
         if cfg!(target_os = "linux") {
+            let executable = if let Some(distro) = self.distro.clone() {
+                match distro.as_str() {
+                    "alpine" => true,
+                    _ => false,
+                }
+            } else {
+                false
+            };
+
             let template = self.supervise_template()?;
             let service_path = self.service_path();
 
@@ -264,10 +358,33 @@ impl<'a> Properties {
                 }
             };
 
+            if executable {
+                let mut perms = std::fs::metadata(service_path.clone())?.permissions();
+                perms.set_mode(0755);
+                std::fs::set_permissions(service_path.clone(), perms)?;
+            }
+
+            let systemd_help = format!("Don't forget to `systemctl daemon-reload`, `systemctl enable zeronsd-{}` and `systemctl start zeronsd-{}`.", self.network, self.network);
+            let alpine_help = format!(
+                "Don't forget to `rc-update add zeronsd-{}` and `rc-service zeronsd-{} start`",
+                self.network, self.network
+            );
+
+            let help = if let Some(distro) = self.distro.clone() {
+                match distro.as_str() {
+                    "alpine" => alpine_help,
+                    _ => systemd_help,
+                }
+            } else {
+                systemd_help
+            };
+
             info!(
-                "Service definition written to {}.\nDon't forget to `systemctl daemon-reload` and `systemctl enable zeronsd-{}`",
-                service_path.to_str().expect("Could not coerce service path to string"),
-                self.network,
+                "Service definition written to {}.\n{}",
+                service_path
+                    .to_str()
+                    .expect("Could not coerce service path to string"),
+                help,
             );
         } else if cfg!(target_os = "macos") {
             let template = self.supervise_template()?;
